;Fang
;by smkdan
;will hang from cieling and fly and circle when mario comes or will fly blindly in a sine wave

;USES THE EXTRA BIT

;If extra bit is clear, it will hang and fly when mario comes.  See (1) for how the extar byte is used
;If it's set, it will be flying in a sine wave.  See (2)

;(1)EXTRA PROP 1 determines the range before the bat will start flying from its hanging spot
;(2)EXTRA PROP 1 determines the Xspeed of the bat while it's flying

PROPRAM = $04
PROPINDEX = $05
XDISPINDEX = $06
YDISPINDEX = $07
FRAMEINDEX = $08
TEMP = $09

SWOOP_SPD	dcb $08,$F8
FLY_SPD		dcb $18,$E8
SPD_CAP		dcb $18,$E8
INCDEC		dcb $01,$FF

EORTBL	dcb $FF,$00
EORTBL2	dcb $00,$FF
EORSINE	dcb $0F,$00

;C2: motion bits
;1570: frame counter / frame decision bit
;1602: Sine index

SINE	dcb $00,$F4,$EA,$E3,$E0,$E3,$EA,$F4 
	dcb $00,$0C,$16,$1D,$20,$1D,$16,$0C

dcb "INIT"
	LDA $7FAB10,x	;Access extra bit
	AND #$04	;low bit
	BNE Dont_Hang	;set = flying
	STZ $1602,x	;reset sine index
	STZ $1570,x	;reset frame to shows
	STZ $C2,x	;reset flying status
	RTL

Dont_Hang
	LDA #$01	;byte to be stored potentially
	STA $1570,x
	STA TEMP	;store to TEMP
	JSR SUB_HORZ_POS
	BNE Start_Left
	DEC TEMP
Start_Left
	LDA TEMP
	STA $157C,x	;face mario
	RTL

dcb "MAIN"
	PHB
	PHK
	PLB
	JSR Run
	PLB
	RTL	;also nothing

Return_l
	RTS

Return_I_l
	JSL $01802A		;speed update
	JSL $01A7DC		;mario interact
	RTS

Run
	JSR SUB_OFF_SCREEN_X0	;offscreen scenario
	JSR GET_DRAW_INFO
	JSR GFX

	LDA $14C8,x		;02 when dieing?
	CMP #$08            	 
	BNE Return_l           
	LDA $9D			;locked sprites?
	BNE Return_l
	LDA $15D0,x		;being eaten by yoshi?
	BNE Return_l

	LDA $7FAB10,x		;operation in extra bit
	AND #$04
	BNE Flying		;go to flying section if not set to hang

	LDA $1570,x
	BEQ Still_Hanging	;go to hanging if zero, else it's flying
	JMP Flying

Still_Hanging
	STZ $AA,x		;if hanging, not moving at all
	STZ $B6,x		;in either direction

	JSR SUB_VERT_POS	;if it's below mario, don't do anything
	BNE Return_I_l

	LDA #$01
	STA TEMP		;Temp byte used to hold the direction to be stored
	JSR SUB_HORZ_POS	;determine where mario is located, Z = 1 when mario is left
	BNE Hanging_Left
	DEC TEMP		;Temp = 0		
Hanging_Left
	LDA TEMP		;store Temp to direction
	STA $157C,x

	LDA $14E0,x		;sprite x high
	XBA
	LDA $E4,x		;sprite x low
	REP #$20		;16bitA
	SEC
	SBC $94			;sub mario x
	SEP #$20		;8bitA
	LDY $157C,x		;load direction byte..
	EOR EORTBL,y		;invert if needed
	CMP $7FAB28,x		;range define by Extra Prop 1
	BCS Return_I_l		;return not within range
	LDA #$01
	STA $1570,x		;not hanging anymore, set 1570

;EXPCODE

Flying
	LDA $7FAB10,x		;load extra bit, how to fly
	AND #$04
	BEQ Was_Hanging		;if it wasn't set to hang...
	JSR Sine_Motion		;practically none of below code applies so it's packed away in its own routine
	JMP Continue		;branch out of range

Was_Hanging
	LDA $C2,x		;load motion bits test bit 0
	CMP #$01		;if set then continue circle motion
	BEQ Slow_Descent	;kerb Yspd
	CMP #$02
	BEQ Doing_Circle
	CMP #$03
	BEQ Fly_Ahead
			
	BRA Not_Circling	;else not circling

Slow_Descent
	LDA $B6,x
	LDY $157C,x		;direction...
	CMP SPD_CAP,y		;cap speed according to direction
	BEQ Dont_SlowX
	CLC
	ADC INCDEC,y		;+1/-1 according to direction
	STA $B6,x		;new Yspd

Dont_SlowX	
	LDA $AA,x		;ramp down Yspd
	SEC
	SBC #$04		;nice steady rise
	STA $AA,x
	BPL Continue		;if it's till sinking just animate...else enter circle  motion
	
	LDA #$02
	STA $C2,x		;halting complete, actual circle motion.  Also enter circle routine motion1

Doing_Circle
	LDA $1602,x		;load sine index

	LDY $157C,x		;load direction
	EOR EORSINE,y		;invert if necessary

	PHA			;preserve sine index
	TAX			;into X
	LDA SINE,x		;load sine byte

	EOR EORTBL,y		;invert as needed

	LDX $15E9		;load sprite index again...
	STA $AA,x		;new Y spd

	PLA			;grab sine index
	CLC
	ADC #$04		;add 4, simulate prescence of a COS table.  quarter circle advance.
	AND #$0F		;needs clipping aswell
	TAX			;into X...
	LDA SINE,x		;new sine byte

	EOR EORTBL,y		;invert as needed

	LDX $15E9		;load sprite index again...
	STA $B6,x		;new X spd

	LDA $14			;frame counter...
	AND #$03
	BNE Continue		;only advance counter on X frames

	LDA $1602,x
	INC A			;advance...
	AND #$0F		;only 16 entries
	STA $1602,x		;new sine index

	BNE Continue		;if not clipped to zero, just animate
	LDA #$03		;else advance motion
	STA $C2,x
	
	BRA Continue	

Fly_Ahead
	LDY $157C,x		;load direction...
	LDA FLY_SPD,y		;load appropriate flying speed
	STA $B6,x		;Xspd

	STZ $AA,x		;no Yspd
	BRA Continue		;and animate
	
Not_Circling
	JSR SUB_VERT_POS	;if it is not above mario, do the circular motion
	BEQ Still_Above

	LDA #$01		;set bit 0 to enter circle motion
	STA $C2,x

	BRA Return_I		;Return??	

Still_Above  			;NEEDS FIX FOR EITHER DIRECTION AND MAYBE SOME DISTANCES
	LDA #$18
	STA $AA,x		;Yspd fixed

	LDY $157C,x		;load direction
	LDA SWOOP_SPD,y		;load appropriate speed
	STA $B6,x		;slower B speed.

;EXPCODE

Continue
	LDA $AA,x		;yspd...
	BMI Flap_Fast		;animate faster if rising, dont skip frames
	LDA $14			;frame counter...
	AND #$03		;every 4th frame
	BNE Return_I		

Flap_Fast
	LDA $1570,x
	AND #$80		;bit 7 = dec or inc.  set if dec
	BEQ Increment

;Decrement
	LDA $1570,x
	DEC A			;decrement frame no.
	STA $1570,x
	CMP #$81
	BNE Return_I
	LDA #$01		;bit 7 clr, inc and 1 for the frame number
	STA $1570,x
	BRA Return_I

Increment
	LDA $1570,x
	INC A			;increment frame no.
	STA $1570,x
	CMP #$04		;frame cap
	BNE Return_I		;dont go above 4
	LDA #$84		;bit 7 = dec, 6 for the frame
	STA $1570,x
	
Return_I
	JSL $01802A		;speed update
	JSL $01A7DC		;mario interact

Return
	RTS

;====

Sine_Motion
	LDA $7FAB34,x		;Extra Prop 2
	LDY $157C,x		;load direction...
	EOR EORTBL2,y		;invert accordingly
	STA $B6,x		;Xspd

	LDA $E4,x		;xscroll
	LSR A
	LSR A			;every 8th frame, change
	AND #$0F		;only 16 entries
	TAY
	LDA SINE,y		;grab entry
	STA $AA,x		;new Yspd

	LDA $14
	AND #$07		;every 8th frame

	BNE Ret_Sine	
Ret_Sine	
	RTS			;nothing for now

;=====

TILEMAP	dcb $C0,$EE,$EE		;hanging

	dcb $C4,$AE,$C4		;wings high
	dcb $C2,$AE,$C2
	dcb $C2,$AE,$C2
	dcb $C4,$AE,$C4		;full stroke


XDISP	dcb $00,$00,$00

	dcb $F5,$00,$0B		;wings high...
	dcb $F3,$00,$0E
	dcb $F3,$00,$0E
	dcb $F5,$00,$0B

	dcb $F4,$00,$0A
	dcb $F2,$00,$0D
	dcb $F2,$00,$0D
	dcb $F4,$00,$0A

YDISP	dcb $FF,$FF,$FF

	dcb $FD,$00,$FD		;Y Disp values apply equally
	dcb $00,$FF,$00
	dcb $FC,$FE,$FC
	dcb $00,$FE,$00

PROP	dcb $00,$00,$00
	
	dcb $40,$00,$00
	dcb $40,$00,$00
	dcb $C0,$00,$80
	dcb $C0,$00,$80		;hanging + right, accesses this one...

	dcb $40,$40,$00
	dcb $40,$40,$00
	dcb $C0,$40,$80
	dcb $C0,$40,$80


COUNT	dcb $00,$02		;A series of tables that I have to get around the fact that tiles MUST be drawn in priority order
CMPTBL	dcb $03,$FF
INCDEC	dcb $01,$FF

GFX
	LDA $15F6,x	;load properties
	STA PROPRAM	;into TEMP

	LDA $1570,x	;load frame to show
	AND #$7F	;clip decision bit
	STA TEMP	;use temp
	ASL A		;x2
	CLC
	ADC TEMP	;x3
	STA FRAMEINDEX	;all 3 bytes per entry
	STA PROPINDEX	;propindex handled seperately

	LDA $157C,x
	STA TEMP	;direction in TEMP
	BNE Facing_Left
	LDA PROPINDEX
	CLC
	ADC #$0C	;12 bytes into other PROP table
	STA PROPINDEX

	LDA $1570,x	;SPECIAL CASE
	BNE Facing_Left	;if not flying yet...
	STZ PROPINDEX	;prop index = 0
	LDA #$40
	TSB PROPRAM	;set mirror for it

Facing_Left
	LDX TEMP
	LDA COUNT,x	;counter goes into X depending on direction
	TAX

OAM_Loop
	TXA
	CLC
	ADC PROPINDEX
	PHX
	TAX
	LDA $00
	CLC
	ADC XDISP,x
	STA $0300,y
	PLX

	TXA
	
	CLC
	ADC FRAMEINDEX
	PHX
	TAX
	LDA $01
	CLC
	ADC YDISP,x
	STA $0301,y
	PLX

	TXA
	CLC
	ADC FRAMEINDEX
	PHX
	TAX
	LDA TILEMAP,x
	STA $0302,y
	PLX

	TXA
	CLC
	ADC PROPINDEX
	PHX
	TAX
	LDA PROPRAM	;sprite properties
	ORA PROP,x	;flip/mirror bits
	ORA $64		;level info
	STA $0303,y
	PLX

	INY
	INY
	INY
	INY

	TXA		;X->A for math
	LDX TEMP	;direction...
	CLC
	ADC INCDEC,x	;-1 or +1
	CMP CMPTBL,x	;and CMP depending on direction
	BEQ Loop_End
	TAX

	BRA OAM_Loop

Loop_End
	LDX $15E9	;restore sprite index
	
	LDY #$02
	LDA #$02	;3 tiles
	JSL $01B7B3

	RTS


;=================
;BORROWED ROUTINES
;=================

;GET_DRAW_INFO
;==============

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
	

;SUB_OFF_SCREEN
;==============

TABLE3              dcb $40,$B0
TABLE6              dcb $01,$FF 
TABLE4              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X0   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | 
SUB_OFF_SCREEN_X1   LDA #$04                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X2   LDA #$02                ;  |
STORE_03            STA $03                 ;  |
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_HB   STZ $03                 ; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_2            ; /    
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;A:0101 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0256 VC:090 00 FL:16953
                    CLC                     ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0280 VC:090 00 FL:16953
                    ADC TABLE4,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0294 VC:090 00 FL:16953
                    ROL $00                 ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0326 VC:090 00 FL:16953
                    CMP $E4,x               ;x pos ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0364 VC:090 00 FL:16953
                    PHP                     ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0394 VC:090 00 FL:16953
                    LDA $1B                 ;x boundry hi ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizCHC:0416 VC:090 00 FL:16953
                    LSR $00                 ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0440 VC:090 00 FL:16953
                    ADC TABLE5,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdizcHC:0478 VC:090 00 FL:16953
                    PLP                     ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizcHC:0510 VC:090 00 FL:16953
                    SBC $14E0,x             ;x pos high ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0538 VC:090 00 FL:16953
                    STA $00                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0570 VC:090 00 FL:16953
                    LSR $01                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0594 VC:090 00 FL:16953
                    BCC LABEL20             ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0632 VC:090 00 FL:16953
                    EOR #$80                ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0648 VC:090 00 FL:16953
                    STA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0664 VC:090 00 FL:16953
LABEL20             LDA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0688 VC:090 00 FL:16953
                    BPL RETURN_2            ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0712 VC:090 00 FL:16953
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KILL_SPRITE         ; /
                    LDY $161A,x             ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0140 VC:071 00 FL:21152
                    CPY #$FF                ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0172 VC:071 00 FL:21152
                    BEQ KILL_SPRITE         ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0188 VC:071 00 FL:21152
                    LDA #$00                ; \ mark sprite to come back    A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0204 VC:071 00 FL:21152
                    STA $1938,y             ; /                             A:FF00 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0220 VC:071 00 FL:21152
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_2            RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS RETURN_2            ; /
                    AND #$01                ;A:0227 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0228 VC:112 00 FL:1142
                    STA $01                 ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0244 VC:112 00 FL:1142
                    TAY                     ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0268 VC:112 00 FL:1142
                    LDA $1C                 ;A:0201 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0282 VC:112 00 FL:1142
                    CLC                     ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0306 VC:112 00 FL:1142
                    ADC TABLE3,y            ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0320 VC:112 00 FL:1142
                    ROL $00                 ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0352 VC:112 00 FL:1142
                    CMP $D8,x               ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0390 VC:112 00 FL:1142
                    PHP                     ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0420 VC:112 00 FL:1142
                    LDA.W $001D             ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNVMXdizcHC:0442 VC:112 00 FL:1142
                    LSR $00                 ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdiZcHC:0474 VC:112 00 FL:1142
                    ADC TABLE6,y            ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdizCHC:0512 VC:112 00 FL:1142
                    PLP                     ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0544 VC:112 00 FL:1142
                    SBC $14D4,x             ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0572 VC:112 00 FL:1142
                    STA $00                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0604 VC:112 00 FL:1142
                    LDY $01                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0628 VC:112 00 FL:1142
                    BEQ LABEL22             ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0652 VC:112 00 FL:1142
                    EOR #$80                ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0668 VC:112 00 FL:1142
                    STA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0684 VC:112 00 FL:1142
LABEL22             LDA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0708 VC:112 00 FL:1142
                    BPL RETURN_2            ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0732 VC:112 00 FL:1142
                    BMI ERASE_SPRITE        ;A:0280 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0170 VC:064 00 FL:1195

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_VERT_POS
; This routine determines if Mario is above or below the sprite.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B829
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_VERT_POS		LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
			LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
			SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
			SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
			STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
			LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
			SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
			BPL SPR_L11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
			INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
SPR_L11			RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
			LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
			SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
			SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
			STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
			LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
			SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
			BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
			INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16			RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642